/*
 * Header file for dealing with Calvin chatserver channels.
 *
 * Part of Chatserv, by Jonathan Lennox.  Written for no particularly good
 * reason, because I felt like it.  Allows multiple people to chat over the
 * net in a relatively straightforward environment.  Disclaimer: I figured out
 * how to do TCP/IP programming by reading man pages.
 *
 * Copyright 1995 Jonathan Lennox
 *
 * See LICENSE for licensing terms.
 */

#include <sys/types.h>

/* In the future, this could become run-time or even dynamically
   configurable. */
/* This number had better be <= the number of bits in an int. */
/* Changed to 32 -- mbg */
#define NUM_CHANNELS (32)

/* The value primary_channel takes when we're not actually talking to
   any channel. */
#define UNCHANNEL (-1)

struct channels {
  int primary_channel;  /* The channel which input will write to */
  struct connection *next[NUM_CHANNELS];
  struct connection *prev[NUM_CHANNELS];
  int channel_map; /* Bitmap representing the channels the user receives input
		      from */
};

struct channel_heads {
  char *desc;
  struct connection *head;
  int connection_count;
  time_t last_nonidle;
};

/* Stuff for "identify_recipient" */
#define RCPT_UNSPEC   1
#define RCPT_USER     RCPT_UNSPEC << 1
#define RCPT_CHANNEL  RCPT_USER << 1
#define RCPT_EVERYONE RCPT_CHANNEL << 1

struct recipient {
  int argtype;
  union {
    struct connection *user;
    int channel;
  } u;
  int explicit;
};

extern struct channel_heads channels[];
extern int default_channel;

int set_up_channels();
int init_connection_channels(struct connection *newcon);
int num_channels();
int valid_channel(int channel);
int primary_channel(struct connection *user);
int join(struct connection *user, int channel);
int leave(struct connection *user, int channel);
int change_channels(struct connection *user, int new_primary, int new_map);
int write_to_channel(char *on_channel, char *off_channel, int channel);
int write_to_channelmap(char *string, int map);
int close_all_channels(struct connection *user);
char *get_channel_topic(int);
int set_channel_topic(char *, int);
int reset_channel_topics(void);
int channel_nonidle(int channel);
struct recipient identify_recipient(int, struct connection *, char *, char **);

/* Commands */
int channel_cmd(char *cmd_name, struct connection *user, char *args);
int join_cmd(char *cmd_name, struct connection *user, char *args);
int leave_cmd(char *cmd_name, struct connection *user, char *args);
int hide_cmd(char *cmd_name, struct connection *user, char *args);
int cloak_cmd(char *cmd_name, struct connection *user, char *args);
int uncloak_cmd(char *cmd_name, struct connection *user, char *args);
int list_cmd(char *cmd_name, struct connection *user, char *args);
int listall_cmd(char *cmd_name, struct connection *user, char *args);
int whois_cmd(char *cmd_name, struct connection *user, char *args);
int users_cmd(char *cmd_name, struct connection *user, char *args);
int topic_cmd(char *cmd_name, struct connection *user, char *args);
