/*
 * Encrypt a password using crypt(3).
 *
 * Not intended for use in high-security situations, e.g. to encrypt login
 * passwords.
 *
 * In general, not for use in "hostile" situations.  It's fragile in several
 * interesting ways.  Primarily intended to generate the encrypted passwords
 * for the RC file for Jon's Chatserver.
 *
 * Part of Chatserv, by Jonathan Lennox.  Written for no particularly good
 * reason, because I felt like it.  Allows multiple people to chat over the
 * net in a relatively straightforward environment.  Disclaimer: I figured out
 * how to do TCP/IP programming by reading man pages.
 *
 * Copyright 1995, 1996 Jonathan Lennox
 *
 * See LICENSE for licensing terms.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#if STDC_HEADERS
# include <string.h>
# include <stdlib.h>
#else
# if HAVE_STRING_H
#  include <string.h>
# else
#  include <strings.h>
# endif
# if HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif
#if HAVE_UNISTD_H
# include <unistd.h>
#endif

#include <time.h>
#include <stdio.h>
#include "crypt.h"
#include "sunos-protos.h"

void to64(char *s, long v, int n);

int main(int argc, char *argv[])
{
  char salt[12], *password, *encrypted;

  if (argc > 1) {
    strncpy(salt, argv[1], 11);
    salt[11] = '\0';
  } else {
    (void)srand((int)time((time_t *)NULL));
    to64(&salt[0], rand(), 2);
    salt[2] = '\0';
  }
  printf("Encrypting with salt \"%s\"\n", salt);
  password = getpass("Password to encrypt: ");
  if (password == NULL)
    exit(1);
  encrypted = crypt(password, salt);
  memset(password, 0, 8);
  printf("Password encrypts to:\n%s\n", encrypted);

  exit(0);
}

unsigned char itoa64[] =         /* 0 ... 63 => ascii - 64 */
        "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";

void to64(char *s, long v, int n)
{
  while (--n >= 0) {
    *s++ = itoa64[v&0x3f];
    v >>= 6;
  }
}
