; calvinhelps.tf -- Macros for use with a Calvin Chatserver
;
; Written by Matthew Gerber <mgerber@morningstar.eyrie.org>
; Modifications by Eagle <eagle@eyrie.org>
;   and Jon Robertson <jonrober@eyrie.org>

/echo % Calvin Chatserver TinyFugue helps Revision: 4.32
/echo % By Matthew Gerber <mgerber@morningstar.eyrie.org>, 
/echo %   Eagle <eagle@eyrie.org>, and Jon Robertson <jonrober@eyrie.org>
/echo % Type /calvinhelp for help with characters and attributes.

/require tools.tf

; ------------------------------- USER-CONFIGURABLE OPTIONS

; If you've got XWindows, change this to yes.

/set ch_xwindows=no

; Always want channel numbers?  Change this to 3.  Want spam protection?
; Change it to 5.  Want both?  Change it to 7.

/set ch_init_client_mode 1

; Does your server have more or fewer than 32 channels?  Change this
; appropriately.

/set ch_high_channel 31

; ------------------------------- CHARACTER HANDLING

; Character handling.  When a character is added, it is associated with a
; prefix.  All sends starting with that prefix are trapped and the text of
; the send rewritten to make it appear to be from that character (using
; standard bracket channeling conventions).  : and ; prefixes are also
; recognized either before or after the prefix, with the standard
; meanings.  Characters are also associated with a channel, so all messages
; from the character go to that channel.  The variable cc_<prefix> is used
; internally to store information about the character.

; /addchar <prefix> <name> <channel>
;
; 	Associates prefix <prefix> with a character named <name> on
;	channel <channel>.

/def -i calvinhelp_addchar = \
  /if /ismacro cc_%1 %; /then \
    /calvinhelp_delchar %1 %; \
  /endif %; \
  /test $[regmatch("^([^ ]+) +(.*) +([^ ]+)$$", {*})] %; \
  /if ({P3} =~ "") %; \
    /echo % You must give a prefix, name and channel for the character! %; \
  /else %; \
    /def -p2 -i -mregexp -h"SEND ^%P1( .*)?$$" ch_char_hook_%1 = \
      /if ({P1} =~ "") %%; /calvinhelp_defaultchar %P1 %%; \
      /else @msg %P3 @as @%P2@ %%P1 %%; \
      /endif %; \
    /echo % Added character %P2 on channel %P3 with nick %P1 %; \
    /def -i cc_%1 = Nick: %P1  Name: %P2  Channel: %P3 %; \
  /endif

; /addnarr <prefix> <channel>
;
; 	Associates prefix <prefix> with narration on channel <channel>.

/def -i calvinhelp_addnarr = \
  /if /ismacro cc_%1 %; /then \
    /calvinhelp_delchar %1 %; \
  /endif %; \
  /test $[regmatch("^([^ ]+) +([^ ]+)$$", {*})] %; \
  /if ({P2} =~ "") %; \
    /echo % You must give a prefix and channel for the narration! %; \
  /else %; \
    /def -p2 -i -mregexp -h"SEND ^%P1( .*)?$$" ch_char_hook_%1 = \
      /if ({P1} =~ "") %%; /calvinhelp_defaultchar %P1 %%; \
      /else @msg %P2 @na %%P1 %%; \
      /endif %; \
    /echo % Added narration prefix %P1 on channel %P2 %; \
    /def -i cc_%1 = Nick: %P1  Channel: %P2 %; \
  /endif

; Only meant to be called by one of the triggers set up by /addchar.
;
;    Sets a character to be the default.  Everything typed that does not begin
;    with @ % [ $ or a space is treated like it has had a prefix added to it.

; We use a glob match here so that we can /calvinhelp_undefchar *.

/def -i calvinhelp_undefchar = \
  /if /ismacro -mglob ch_prefix_%1 %; /then \
    /purge -i -mglob ch_prefix_%1 %; \
    /purge -i -mglob ch_defaultchar_%1 %; \
    /set ch_defchar= %; \
    /set ch_defcharchan= %; \
    /echo % Default character prefix has been turned off. %; \
  /endif

/def -i calvinhelp_makedefchar = \
  /def -p1 -i -mregexp -h"SEND ^[^@%%$$\\[/]" ch_prefix_%1 = \
    /if (substr({P0}, 0, 1) =~ " ") \
      /send %%* %%; \
    /else \
      %1 %%* %%; \
    /endif %; \
  /echo % %1 is the default character prefix. %; \
  /test $[regmatch("^Nick: .+  Name: (.+)  Channel: (.+)$$", ${cc_%1})] %; \
  /if ({P1} =~ "") \
     /test $[regmatch("^Nick: .+  Channel: (.+)$$", ${cc_%1})] %; \
     /set ch_defchar=<Narrate> %; \
     /set ch_defcharchan=: %P1 %; \
  /else \
     /set ch_defchar=%P1 %; \
     /set ch_defcharchan=: %P2 %; \
  /endif %; \
  /def -i ch_defaultchar_%1 = %1 is the default character.

/def -i calvinhelp_defaultchar = \
  /if /ismacro ch_prefix_%1 %; /then \
    /calvinhelp_undefchar %1 %; \
  /elseif /ismacro ch_prefix_* %; /then \
    /calvinhelp_undefchar * %; \
    /calvinhelp_makedefchar %1 %; \
  /else %; \
    /calvinhelp_makedefchar %1 %; \
  /endif

; /delchar <prefix>
;
;	Removes the character associated with <prefix>.  Can accept globs.

/def -i calvinhelp_delchar = \
  /if /ismacro ch_defaultchar_%1 %; /then %1 %; /endif %; \
  /if /ismacro -mglob cc_%1 %; /then \
    /purge -i -mglob ch_char_hook_%1 %; \
    /purge -i -mglob cc_%1 %; \
    /echo % Removed character %1 %; \
  /endif

; /listchars
;
;	Lists all defined characters.

/def -i calvinhelp_listchars = \
  /echo % Defined characters: %; \
  /list -i cc_* %; \
  /list -i ch_defaultchar_*

; ------------------------------- REFLECTOR AND CHARACTER REWRITING

; Rewrite to remove the numerics.
/def -i -F -p1 -mregexp -t"\\\1" unescape_one = \
   /test substitute(replace("\\1", "\\\\", {*}))
/def -i -F -p1 -mregexp -t"\\\2" unescape_two = \
   /test substitute(replace("\\2", "|", {*}))

/def -i calvinhelp_dosubs = \
   /def -i -F -p3 -mregexp -t"^[0-9]+ (.*)$$" no_sub = \
      /substitute -- %%{P1} %; \
   /def -i -F -p4 -mregexp -t"^[0-9]+ (.*)\\|.(.*)\\|E(.*)$$" one_sub = \
      /substitute -- %%{P1}%%{P2}%%{P3} %; \
   /def -i -F -p5 -mregexp -t"^[0-9]+ (.*)\\|.(.*)\\|E(.*)\\|.(.*)\\|E(.*)$$" two_sub = \
      /substitute -- %%{P1}%%{P2}%%{P3}%%{P4}%%{P5} %; \
   /def -i -F -p6 -mregexp -t"^[0-9]+ (.*)\\|.(.*)\\|E(.*)\\|.(.*)\\|E(.*)\\|.(.*)\\|E(.*)$$" three_sub = \
      /substitute -- %%{P1}%%{P2}%%{P3}%%{P4}%%{P5}%%{P6}%%{P7} %; \
   /def -i -F -p7 -mregexp -t"^[0-9]+ (.*)\\|.(.*)\\|E(.*)\\|.(.*)\\|E(.*)\\|.(.*)\\|E(.*)\\|.(.*)\\|E(.*)$$" four_sub = \
      /substitute -- %%{P1}%%{P2}%%{P3}%%{P4}%%{P5}%%{P6}%%{P7}%%{P8}%%{P9}

/def -i -ag -mglob -p200 -t"390[0-3] *" gag_client_msg

; Reflector rewriting:
/def -i -F -p103 -mregexp \
  -t"^10[02]1 <(\\|c[0-9]+\\|E: )?\\|n=R=\\|E> \\|1\\*\\*\\* (.*)\\|E$$" ref_server = \
    /substitute 9999 *** (Ref) %{P2}


; Catch and possibly rewrite in-character lines.  If we're eagtrusting a
; channel, we rewrite to be indistinguishable from a normal user's statement.

; NOTE:  In-character messages from off-channel are deprecated as of
; Calvinhelps 4.0.

/def -i -p102 -F -mregexp \
 -t"^10([02])5 \\* (\\|c([0-9]+)\\|E: )?(\\|1.*\\|E) \[\\|n([^ ]+)\\|E\]$$" \
 ch_narrate_catch = \
   /if ({P3} =~ "") \
      /let ch_ch=%{ch_mydef} %; \
   /else %; \
      /let ch_ch=%{P3} %; \
   /endif %; \
   /if /ismacro ch_eagtrusting%{ch_ch} %; /then \
      /substitute 10%{P1}5 * %{P2}%{P4} %; \
   /elseif /ismacro ch_untrusting%{ch_ch} %; /then \
      /substitute 10%{P1}1 <%{P2}|n%{P5}|E> [%{P4}]%; \
   /endif


/def -i -p101 -F -mregexp \
 -t"^10([02])[78] \\* (\\|c([0-9]+)\\|E: )?\\|2(.+)\\|E( )?(\\|1.*\\|E) \[\\|n([^ ]+)\\|E\]$$" \
 ch_pose_catch = \
   /if ({P3} =~ "") \
      /let ch_ch=%{ch_mydef} %; \
   /else %; \
      /let ch_ch=%{P3} %; \
   /endif %; \
   /if /ismacro ch_eagtrusting%{ch_ch} %; /then \
      /if ({P5} =~ " ") \
         /substitute 10%{P1}2 * %{P2}|n%{P4}|E %{P6} %; \
      /else %; \
         /substitute 10%{P1}3 * %{P2}|n%{P4}|E%{P6} %; \
      /endif %; \
   /elseif /ismacro ch_untrusting%{ch_ch} %; /then \
      /if ({P5} =~ " ") \
         /substitute 10%{P1}1 <%{P2}|n%{P7}|E> [|1%{P4}|E %{P6}]%; \
      /else %; \
         /substitute 10%{P1}1 <%{P2}|n%{P7}|E> [|1%{P4}|E%{P6}]%; \
      /endif %; \
   /endif


/def -i -p100 -F -mregexp \
 -t"^10([02])6 <(\\|c([0-9]+)\\|E: )?\\|2(.+)\\|E> (\\|1.*\\|E) \[\\|n([^ ]+)\\|E\]$$" \
 ch_say_catch = \
   /if ({P3} =~ "") \
      /let ch_ch=%{ch_mydef} %; \
   /else %; \
      /let ch_ch=%{P3} %; \
   /endif %; \
   /if /ismacro ch_eagtrusting%{ch_ch} %; /then \
      /substitute 10%{P1}1 <%{P2}|n%{P4}|E> %{P5}%; \
   /elseif /ismacro ch_untrusting%{ch_ch} %; /then \
      /substitute 10%{P1}1 <%{P2}|n%{P6}|E> [%{P4}] %{P5}%; \
   /endif


/def -i calvinhelp_dotrust = \
  /let ch_trustlist=%-2 %; \
  /let ch_i=0 %; \
  /while (ch_i <= {ch_high_channel}) \
    /if (ch_trustlist =/ (strcat("*{", ch_i, "}*"))) \
      /if ({2} =~ "on") \
        /if /!ismacro ch_%{1}trusting%{ch_i} %; /then \
          /def -i ch_%{1}trusting%{ch_i} = Yes. %; \
        /endif %; \
      /elseif ({2} =~ "off") \
        /if /ismacro ch_%{1}trusting%{ch_i} %; /then \
          /undef ch_%1trusting%{ch_i} %; \
        /endif %; \
      /endif %; \
    /endif %; \
    /let ch_i=$[ch_i+1] %; \
  /done %; \
  /unset ch_i %; \
  /unset ch_trustlist %; \
  /calvinhelp_showtrust %1

/def -i calvinhelp_showtrust = \
  /let ch_s=%% Now %1trusting channels:  %; \
  /let ch_i=0 %; \
  /while (ch_i <= {ch_high_channel}) \
    /if /ismacro ch_%{1}trusting%{ch_i} %; /then \
      /let ch_s=%{ch_s} %{ch_i} %; \
    /endif %; \
    /let ch_i=$[ch_i+1] %; \
  /done %; \
  /unset ch_i %; \
  /echo %ch_s %; \
  /unset ch_s
 
/def -i calvinhelp_showalltrust = \
  /calvinhelp_showtrust eag %; \
  /calvinhelp_showtrust un

/def -i calvinhelp_dotrustall = \
  /let ch_s= %; \
  /let ch_i=0 %; \
  /while (ch_i <= {ch_high_channel}) \
    /let ch_s=%{ch_s} %{ch_i} %; \
    /let ch_i=$[ch_i+1] %; \
  /done %; \
  /calvinhelp_dotrust %1 %2 %ch_s %; \
  /unset ch_s

/def -i calvinhelp_dountrust = /calvinhelp_dotrust un on %*
/def -i calvinhelp_doeagtrust = /calvinhelp_dotrust eag on %*
/def -i calvinhelp_donountrust = /calvinhelp_dotrust un off %*
/def -i calvinhelp_donoeagtrust = /calvinhelp_dotrust eag off %*
/def -i calvinhelp_untrustall = /calvinhelp_dotrustall un on
/def -i calvinhelp_eagtrustall = /calvinhelp_dotrustall eag on
/def -i calvinhelp_untrustnone = /calvinhelp_dotrustall un off
/def -i calvinhelp_eagtrustnone = /calvinhelp_dotrustall eag off

; ------------------------------- CHANNEL ACTVITY HANDLING

/def -i calvinhelp_activity = \
   /let ch_active_list=%-1 %; \
   /let ch_i=0 %; \
   /while (ch_i <= {ch_high_channel}) \
      /if (ch_active_list =/ (strcat("*{", ch_i, "}*"))) \
         /if (({1} =~ "on") & \
              (ch_show_activity !/ (strcat("*{", ch_i, "}*")))) \
            /set ch_show_activity=%{ch_show_activity} %{ch_i} %; \
         /elseif (({1} =~ "off") & \
                  (ch_show_activity =/ (strcat("*{", ch_i, "}*")))) \
            /set ch_show_activity=$(/remove %{ch_i} %{ch_show_activity}) %; \
            /calvinhelp_clearactivity %ch_i %; \
         /endif %; \
      /endif %; \
      /let ch_i=$[ch_i+1] %; \
   /done %; \
   /unset ch_i %; \
   /calvinhelp_showactive

/def -i calvinhelp_showactive = \
   /let ch_i=0 %; \
   /let ch_s=%% Now showing activity for channels:%{ch_show_activity} %; \
   /echo %ch_s
 
/def -i calvinhelp_activityall = \
   /let ch_s= %; \
   /let ch_i=0 %; \
   /while (ch_i <= {ch_high_channel}) \
      /let ch_s=%{ch_s} %{ch_i} %; \
      /let ch_i=$[ch_i+1] %; \
   /done %; \
   /calvinhelp_activity %1 %ch_s %; \
   /unset ch_i

/def -i calvinhelp_clearactivity = \
   /if ({1} =~ "*") \
      /set ch_active_channels=Act: %; \
   /else \
      /let ch_tr=%{1} %; \
      /if ({ch_tr} = "") \
         /let ch_tr="." %; \
      /endif %; \
      /set ch_active_channels=$(/remove %{ch_tr} %{ch_active_channels}) %; \
      /set ch_active_channels=$[substr({ch_active_channels}, 1)] %; \
   /endif

/def -i ca = /calvinhelp_clearactivity %*
/def -i clearactivity = /calvinhelp_clearactivity %*
/def -i showactive = /calvinhelp_showactive %*
/def -i activityall = /calvinhelp_activityall on %*
/def -i activitynone = /calvinhelp_activityall off %*
/def -i activity = /calvinhelp_activity on %* 
/def -i noactivity = /calvinhelp_activity off %*

; ------------------------------- ATTRIBUTE HANDLING

; /chanattr <channel> <attribute>
;
;	Assign attribute <attribute> to messages on channel <channel>.

/def -i calvinhelp_chanattr = \
  /test $[regmatch("^([0-9]+) +([^ ]+) *$$", {*})] %; \
  /if ({P1} =~ "") %; \
    /echo % You must give a channel and an attribute. %; \
  /else %; \
    /if /ismacro cca_%1 %; /then \
      /calvinhelp_nochanattr %1 %; \
    /endif %; \
    /def -i -mglob -F -p100 -a%2 -t"10[23]? *|c%1|E: *" ch_col_%1 %; \
    /echo % Attribute for channel %1 is now %2 %; \
    /def -i cca_%1 = %2 %; \
  /endif

; /nochanattr <channel>
;
;	Removes any attribute from channel number <channel>.

/def -i calvinhelp_nochanattr = \
  /if /ismacro cca_%1 %; /then \
    /undef ch_col_%1 %; \
    /echo % Removed attribute for channel %1 %; \
    /undef cca_%1 %; \
  /endif

; /setattrs <normal> <private> <yell> <server>
;
;	Sets the attribute for normal mesages to <normal>, private messages
;	to <private>, yells to <yell>, and server messages to <server>.


/def -i calvinhelp_setattrs = \
  /if /ismacro c_a %; /then \
    /calvinhelp_noattrs %; \
  /endif %; \
  /def -i -mglob -F -p100 -a%1 -t"10[01][0-9] *" ch_attr_defchan %; \
  /def -i -mglob -F -p100 -a%2 -t"11[01][0-9] *" ch_attr_priv %; \
  /def -i -mglob -F -p100 -a%3 -t"120[0-9] *" ch_attr_yell %; \
  /def -i -mglob -F -p100 -a%4 -t"[2-9][0-9][0-9][0-9] *" ch_attr_sys_numeric %;\
  /def -i -mglob -F -p100 -a%4 -t"\\*\\*\\**" ch_attr_sys_no_numeric %; \
  /echo % Attributes are now:  Def:%1 Prv:%2 Yel:%3 Sys:%4 %; \
  /def -i c_a = Def:%1 Prv:%2 Yel:%3 Sys:%4

; /noattrs
;
;	Removes attributes for normal, private, and server messages and
;	yells.

/def -i calvinhelp_noattrs = \
  /if /ismacro c_a %; /then \
    /undef ch_attr_defchan %; \
    /undef ch_attr_priv %; \
    /undef ch_attr_yell %; \
    /undef ch_attr_sys_numeric %; \
    /undef ch_attr_sys_no_numeric %; \
    /echo % Attributes removed. %; \
    /undef c_a %; \
  /endif

; /listattrs
;
;	List main and channel attributes.

/def -i calvinhelp_listattrs = \
  /echo % Main attributes: %; \
  /list -i c_a %; \
  /echo % Channel attributes: %; \
  /list -i cca_*

; ------------------------------- GAGGING

; /addgag <player> [<channel>]
;
;	Adds a gag on player/character <player> on channel <channel>.
;	If <channel> isn't specified, assumes the default channel.

/def -i calvinhelp_addgag = \
  /if ({2} =~ "") \
    /def -i -ag -mglob -p200 \
      -t"10[01][1-9] *\\|n%1\\|E*" gag_def_player_%1 %; \
    /def -i -ag -mglob -p200 \
      -t"100[6-8] *\\|2%1\\|E*" gag_def_alias_%1 %; \
    /echo % Gagged player/character %1 on the default channel. %; \
  /else \
    /def -i -ag -mglob -p200 \
      -t"10[23][1-9] *\\|c%2\\|E*\\|n%1\\|E*" gag_%2_player_%1 %; \
    /def -i -ag -mglob -p200 \
      -t"102[6-8] *\\|c%2\\|E*\\|2%1\\|E*" gag_%2_alias_%1 %; \
    /echo % Gagged player/character %1 on channel %{2}. %; \
  /endif

/def -i calvinhelp_gagall = \
   /def -i -ag -mglob -p200 \
      -t"1*\\|n%1\\|E*" gag_all_%1 %; \
   /echo % Gagged all utterances from player %1.

/def -i calvinhelp_delgagall = \
   /undef gag_all_%1 %; \
   /echo % Ungagged all utterances from player %1.

/def -i calvinhelp_gagmsg = \
   /def -i -ag -mglob -p200 \
      -t"11*\\|n%1\\|E*" gag_msg_%1 %; \
   /echo % Gagged private messages from player %1.

/def -i calvinhelp_delgagmsg = \
   /undef gag_msg_%1 %; \
   /echo % Ungagged private messages from player %1.

; /delgag <player> [<channel>]
;
;	Removes a gag on player or character <player> on channel
;	<channel>.  If <channel> isn't specified, assumes the default
;	channel.

/def -i calvinhelp_delgag = \
  /if (%2 =~ "") \
    /undef gag_def_player_%1 %; \
    /undef gag_def_alias_%1 %; \
    /echo % Ungagged player/character %1 on the default channel. %; \
  /else \
    /undef gag_%2_player_%1 %; \
    /undef gag_%2_alias_%1 %; \
    /echo % Ungagged player/character %1 on channel %{2}. %; \
  /endif

; ------------------------------- RECALL

; Recall commands that allow you to recall specified channels or whispers.
; In all cases, <lines in history> refers to the lines that we search
; through and not the total lines that are displayed.

; /recchan <channel #> <lines in history>
;
;    Recalls only those lines in the history that are from the specified
;    channel number.

/def -i calvinhelp_recallchannel = \
  /recall -mregexp /%2 ^((## |[\*] [[{]?|[[<{])%1: )

; /recwhisp <lines in history>
;
;    Recalls only those lines in the history that are whispers.

/def -i calvinhelp_recallwhisper = \
  /recall -mregexp /%1 ^([\*][\*][\*][\*]+|->|[\*]([^ \*]| ->)|##(>| ->))

; /recwhispname <name> <lines in history>
;
;    Recalls only those lines in the history that are whispers from the
;    specified name.  <name> is case sensitive.

/def -i calvinhelp_recallwhispername = \
  /recall -mregexp /%2 ^((-> )?[\*]%1([0-9]|[_+]|@.+)?[\*]|([\*]|##)( -)?> %1([0-9]|[_+]|@.+)?(:| ))

; /recdef <lines in history>
;
;    Recalls only those lines in the history that are from the default
;    channel.  This version does not recall yells.

/def -i calvinhelp_recalldefault = \
  /recall -mregexp /%1 ^([[<{]|[\*] [{[]?)(([^{[0-9-]|-[^>]|->[^ ])|([^-{[0-9]|[0-9][^0-9:]|[0-9][0-9][^:]))

; /recchanwrap <width> <channel #> <lines in history>
;
;    Recalls lines from a specified channel and wraps them to a specified
; width.  Useful for spams.

/def -i calvinhelp_recchanwrap = \
   /let ch_ws=%{wrapsize} %; \
   /set wrapsize=%1 %; \
   /calvinhelp_recallchannel %2 %3 %; \
   /set wrapsize=%{ch_ws}

; ------------------------------- SERVER STATES

; Catch starting nick query and turn on @client 1.
/def -i -p10 -mregexp -h"PROMPT ^\\*\\*\\* Please specify a nickname:" \
  -aBCwhite ch_donumerics = /send @client %{ch_init_client_mode}

; Catch "You are now known as", and do cleanup.

/def -i calvinhelp_login = \
  /if /ismacro ch_catchprompt %; /then /undef ch_catchprompt %; /endif %; \
  /set ch_mydef= %; \
  /set ch_channels= %; \
  /unset ch_autonick %; \
  /if ({ch_autochan} !~ "") \
    @join %ch_autodef %ch_autochan %; \
  /endif %; \
  /unset ch_autochan

; Set the nick-entry trigger for autoconnect.

/def -i calvinhelp_setcontrig = \
   /def -mregexp -h"PROMPT ^8001 " -aBCwhite -i -p10 ch_catchprompt = \
      /if ({ch_autonick} !~ "") \
         /send %%ch_autonick %%; \
         /set ch_autonick=%%{ch_autonick}_ %%; \
      /endif

; Reconnect.

/def -i calvinhelp_reconnect = \
   /set ch_autochan=%{ch_channels} %; \
   /set ch_autodef=%{ch_mydef} %; \
   /if ({ch_autodef} =~ "N") /set ch_autodef=-1 %; /endif %; \
   /set ch_autonick=%{ch_mynick} %; \
   /calvinhelp_setcontrig %; \
   /connect %*

; Autoconnect.

/def -i calvinhelp_autologin = \
  /set ch_autonick=%2 %; \
  /if ({ch_autonick} =~ "") \
    /echo % You must specify a nickname to autoconnect with! %; \
  /else %; \
    /set ch_autodef=%3 %; \
    /if ({ch_autodef} =~ "") \
      /set ch_autodef=-1 %; \
    /endif %; \
    /set ch_autochan=%-2 %; \
    /calvinhelp_setcontrig %; \
    /connect %1 %; \
  /endif

; Keep track of what world we're in.

/def -i -mregexp -hWORLD ch_catchworld = /set ch_world=%*

; Do stuff on disconnect.
/def -i -p200 -F -h'DISCONNECT' ch_discon = \
   /set ch_active_channels=Act:

; Keep track of my nick.

/def -i -mregexp -p10 -F \
  -t"^3203 .*\\|n([^ ]+)\\|E" ch_first_nick = \
   /set ch_mynick=%P1 %; \
   /set ch_mynick=$(/replace \\1 \\ %{ch_mynick}) %; \
   /set ch_mynick=$(/replace \\2 | %{ch_mynick}) %; \
   /if ({ch_xwindows} =~ "yes") \
      /xtitle TinyFugue: %{ch_mynick} on %{ch_world} %; \
   /endif %; \
   /calvinhelp_login

/def -i -mregexp -p10 -F \
  -t"^3204 .+\\|1([^ ]+)\\|E.+\\|n([^ ]+)\\|E" ch_change_nick = \
   /let ch_mn=$(/replace \\1 \\ %{P1}) %; \
   /let ch_mn=$(/replace \\2 | %{ch_mn}) %; \
   /if ({ch_mn} =~ {ch_mynick}) \
      /set ch_mynick=%P2 %; \
      /set ch_mynick=$(/replace \\1 \\ %{ch_mynick}) %; \
      /set ch_mynick=$(/replace \\2 | %{ch_mynick}) %; \
      /if ({ch_xwindows} =~ "yes") \
         /xtitle TinyFugue: %{ch_mynick} on %{ch_world} %; \
      /endif %; \
   /endif


; Detect my default channel changing and trigger on it.

/def -i -mregexp -p10 -F -t"^300[35] .*\\|c([0-9]+)\\|E" ch_set_default = \
  /set ch_mydef=%P1

/def -i -mregexp -p10 -F -t"^300[46]" ch_set_nodefault = /set ch_mydef=N

; Detect myself joining and leaving a channel.

/def -i -mregexp -p10 -F \
  -t"^2003 .+\\|n([^ ]+)\\|E.+\\|c([0-9]+)\\|E" ch_leave = \
   /let ch_mn=$(/replace \\1 \\ %{P1}) %; \
   /let ch_mn=$(/replace \\2 | %{ch_mn}) %; \
   /if ({ch_mn} =~ {ch_mynick}) \
      /set ch_channels=$(/remove %P2 %ch_channels) %; \
      /set ch_active_channels=$(/remove %{P2} %{ch_active_channels}) %; \
      /set ch_active_channels=$[substr({ch_active_channels}, 1)] %; \
   /endif
   
/def -i -mregexp -p10 -F \
  -t"^2004 .+\\|n([^ ]+)\\|E.+\\|c([0-9]+)\\|E" ch_join = \
   /let ch_mn=$(/replace \\1 \\ %{P1}) %; \
   /let ch_mn=$(/replace \\2 | %{ch_mn}) %; \
   /if ({ch_mn} =~ {ch_mynick}) \
      /set ch_channels=%{ch_channels} %{P2} %; \
   /endif

; Detecting activity on a channel.
/def -i -mregexp -p110 -F \
  -t"^10[0-3][0-9] [^\\|]+(\\|c([0-9]+)\\|E)?.*\\|n([^ ]+)\\|E" ch_activity = \
   /let ch_tmpchan=%{P2} %; \
   /if ({ch_tmpchan} =~ "") /let ch_tmpchan=%{ch_mydef} %; /endif %; \
   /let ch_mn=$(/replace \\1 \\ %{P3}) %; \
   /let ch_mn=$(/replace \\2 | %{ch_mn}) %; \
   /if ({ch_mn} =~ {ch_mynick}) \
      /calvinhelp_clearactivity %{ch_tmpchan} %; \
   /else \
      /if (({ch_active_channels} !/ (strcat("*{", ch_tmpchan, "}*"))) & \
           ({ch_show_activity} =/ (strcat("*{", ch_tmpchan, "}*")))) \
         /set ch_active_channels=%{ch_active_channels} %{ch_tmpchan} %; \
      /endif %; \
   /endif


; ------------------------------- STATUS LINE

/set ch_mynick=
/set ch_mydef=N
/set ch_defcharchan=
/set ch_defchar=
/set ch_active_channels=Act:

;More__92_Complex____Jonrober: 15_Construc: 12_(Read)(Active: 2)(Log)(Over)_07:20
/def -i calvinhelp_setstatnickchar = /set status_fields @more:8:Br :1 @world ch_mynick:-8 ": ":2 ch_mydef:2 :1 ch_defchar:-8:Ccyan ch_defcharchan:4:Ccyan :1 @read:6:Ccyan @active:11:Br @log:5 insert:6 :1 @clock:5

;More__92_Complex_______Construction Guy: 12___(Read)(Active: 2)(Log)(Over)_07:20
/def -i calvinhelp_setstatchar = /set status_fields @more:8:Br :1 @world ch_defchar:-16:Ccyan ch_defcharchan:4:Ccyan :1 @read:6:Ccyan @active:11:Br @log:5 insert:6 :1 @clock:5

;More__92_Complex______________________Van: N__(Read)(Active: 2)(Log)(Over)_07:20
/def -i calvinhelp_setstatnick = /set status_fields @more:8:Br :1 @world ch_mynick:-18 ": ":2 ch_mydef:2 :1 @read:6:Ccyan @active:11:Br @log:5 insert:6 :1 @clock:5

;More__92_Complex_________Act: 3 4_____________(Read)(Active: 2)(Log)(Over)_07:20
/def -i calvinhelp_setstatact = /set status_fields @more:8:Br :1 @world ch_active_channels:20:Cred :1 @read:6:Ccyan @active:11:Br @log:5 insert:6 :1 @clock:5

/def -i calvinhelp_setstatnickact = /set status_fields @more:8:Br :1 @world ch_mynick:-8 ": ":2 ch_mydef:2 :1 ch_active_channels:12:Cred :1 @read:6:Ccyan @active:11:Br @log:5 insert:6 :1 @clock:5

/def -i calvinhelp_setstatcharact = /set status_fields @more:8:Br :1 @world ch_defchar:-8:Ccyan ch_defcharchan:4:Ccyan :1 ch_active_channels:12:Cred :1 @read:6:Ccyan @active:11:Br @log:5 insert:6 :1 @clock:5

/def -i calvinhelp_setstatnickcharact = /set status_fields @more:8:Br :1 @world ch_mynick:-8 ": ":2 ch_mydef:2 :1 ch_defchar:-8:Ccyan ch_defcharchan:4:Ccyan :1 ch_active_channels:11:Cred :1 @read:6:Ccyan @log:5 insert:6 :1 @clock:5

/def -i calvinhelp_setstatdef = /set status_fields @more:8:Br :1 @world "Ch: ":4 ch_mydef:2 :1 @read:6:Ccyan @active:11:Br @log:5 insert:6 :1 @clock:5

/def -i calvinhelp_setstatx = /set status_fields @more:8:Br :1 "(":1 ch_mydef:2 ")":1 :1 ch_active_channels::Cred :1 ch_defchar:-10:Ccyan ch_defcharchan:4:fCcyan :1 @read:6:fCBlue @active:11:Br @log:5 insert:6

; ------------------------------- HELP
/def -i calvinhelp_calvinhelp = \
  /echo %; \
  /echo -ah Help on Calvin TinyFugue helps: %; \
  /echo %; \
  /echo -ah Attribute commands (/chattrs): %; \
  /echo     /setattrs, /noattrs, /chanattr, /nochanattr, /listattrs %; \
  /echo %; \
  /echo -ah Character commands (/chchars): %; \
  /echo     /addchar, /delchar, /listchars %; \
  /echo %; \
  /echo -ah Recall commands (/chrecs): %; \
  /echo     /recchan, /recchanwrap, /recdef, /recwhisp, /recwhispname %; \
  /echo %; \
  /echo -ah Activity tracking commands (/chact): %; \
  /echo     /ca, /clearactivity, /showactive, /activityall, /activitynone, %; \
  /echo     /[no]activity %; \
  /echo %; \
  /echo -ah In-character trust commands (/chtrust): %; \
  /echo     /[no]eagtrust, /[no]untrust, /eagtrustall, /eagtrustnone, %; \
  /echo     /untrustall, /untrustnone, /showtrust %; \
  /echo %; \
  /echo -ah Reconnection commands (/chrecon): %; \
  /echo     /recon, /autocon %; \
  /echo %; \
  /echo -ah Status line commands (/chstats): %; \
  /echo     /setstat[nick][char][act], /setstatdef, /setstatx %; \
  /echo %; \
  /echo -ah Gag commands (/chgag): %; \
  /echo     /addgag, /delgag, /[del]gagall, /[del]gagmsg %; \
  /echo

/def -i calvinhelp_chact = \
  /echo %; \
  /echo -ah /ca <channel>, /clearactivity <channel> %; \
  /echo -ah /showactive %; \
  /echo -ah /activityall, /activitynone %; \
  /echo -ah /[no]activity [channel ...] %; \
  /echo %; \
  /echo These commands affect channel activity tracking.  /activityall and %; \
  /echo /activitynone turn tracking on and off for all channels respect- %; \
  /echo ively, /[no]activity turns it off or on for specific channels, and %; \
  /echo /showactive displays the channels for which it's on.  /ca and %; \
  /echo /clearactivity are synonymous; they clear the activity flag for a %; \
  /echo specific given channel. %; \
  /echo

/def -i calvinhelp_chgag = \
  /echo %; \
  /echo -ah /addgag <player> [channel] %; \
  /echo -ah /delgag <player> [channel] %; \
  /echo -ah /[del]gagall <player> %; \
  /echo -ah /[del]gagmsg <player> %; \
  /echo %; \
  /echo These commnds gag messages from a player from your sight, though %; \
  /echo they will still show up in any logs you have.  /addgag gags for %; \
  /echo a channel, or the default if none is specified.  /gagmsg gags all %; \
  /echo whispers from a player.  And /gagall gags everything that isn't a %; \
  /echo server message from the player.  The "del" counterparts remove a %; \
  /echo gag. %; \
  /echo

/def -i calvinhelp_chtrust = \
  /echo %; \
  /echo -ah /[no]eagtrust [channel ...] %; \
  /echo -ah /eagtrustall, /eagtrustnone %; \
  /echo -ah /[no]untrust [channel ...] %; \
  /echo -ah /untrustall, /untrustnone %; \
  /echo -ah /showtrust %; \
  /echo %; \
  /echo These options turn on or off the "trust character messages" %; \
  /echo options for individual channels.  When these options are on, %; \
  /echo in-character messages are reformatted to look nicer.  "eag" %; \
  /echo formatting displays the in-character message as though a real %; \
  /echo person had typed it, removing the [Nick] at the end of the line. %; \
  /echo "un" formatting displays the in-character message as though the %; \
  /echo server did no rewriting on it at all.  /showtrust displays which %; \
  /echo channels are being trusted. %; \
  /echo %; \
  /echo

/def -i calvinhelp_chchars = \
  /echo %; \
  /echo -ah /addchar <local name> <character name> <channel #> %; \
  /echo -ah /addnarr <local name> <channel #> %; \
  /echo -ah /delchar <local name> %; \
  /echo -ah /listchars %; \
  /echo %; \
  /echo /addchar defines a Calvin character and binds them to a channel. %; \
  /echo /addnarr defines a narration alias and binds it to a channel. %; \
  /echo /delchar removes a currently defined character. %; \
  /echo /listchars lists all the characters you have defined. %; \
  /echo %; \
  /echo %; \
  /echo -ah Using defined characters %; \
  /echo %; \
  /echo To have a character speak or pose, simply type their prefix followed%;\
  /echo by the message or pose as you would normally type it, including the %;\
  /echo colon for a pose.  Typing the prefix alone makes everything sent %;\
  /echo afterward that does not begin with another prefix, space, \%, \@, %;\
  /echo \$, or [ be sent as if the prefix was typed.  Starting a line with a %;\
  /echo space causes that line to be sent as if the auto-prefixing was not %;\
  /echo on.  Auto-prefixing turns off when the prefix is retyped.  Typing %;\
  /echo another prefix by itself switches auto-prefixing to that prefix. %;\
  /echo %; \
  /echo Narration aliases work the same, but send narration instead of a %:\
  /echo character's say or pose. %; \
  /echo %; \
  /echo Example:                       (output) %;\
  /echo /addchar jb JimBob 2           (Spew about the defined macros) %;\
  /echo /addchar jj JimJoe 2 %;\
  /echo /addnarr na 2 %;\
  /3cho na It was a dark night.        [It was a dark night.] %;\
  /echo jb Hi all!                     [JimBob] Hi all! %;\
  /echo jb :grins.                     [JimBob grins.] %;\
  /echo jb ;'s watch beeps.            [JimBob's watch beeps.] %; \
  /echo jb                             (prefix is set) %; \
  /echo Hi all!                        [JimBob] Hi all! %;\
  /echo :grins.                        [JimBob grins.] %;\
  /echo @msg joe Yay!                  -> *joe* Yay! (prefix doesn't apply) %; \
  /echo jj                             (prefix is reset) %;\
  /echo ;'s watch beeps.               [JimJoe's watch beeps.] %;\
  /echo jb :chuckles.                  [JimBob chuckles.] %;\
  /echo  This starts with a space.     <You> This starts with a space. %;\
  /echo [[Phone call, BRB.]]           <You> [[Phone call, BRB.]] %;\
  /echo jj                             (prefix is removed) %;\
  /echo Hey everyone.                  <You> Hey everyone. %;\
  /echo

/def -i calvinhelp_chattrs = \
  /echo %; \
  /echo -ah /setattrs <defchan> <privmsg> <yell> <sysmsg> %; \
  /echo -ah /noattrs %; \
  /echo %; \
  /echo /setattrs defines attributes for your default channel, private %; \
  /echo messages and descriptions, yells, and system messages.  Attributes %; \
  /echo must be valid TinyFugue attributes (type /help attributes for %; \
  /echo details).  /noattrs undefines these attributes. %; \
  /echo %; \
  /echo -ah /chanattr <channel #> <attribute> %; \
  /echo -ah /nochanattr <channel #> %; \
  /echo %; \
  /echo /chanattr defines the attribute for a specific channel. /nochanattr%;\
  /echo undefines this attribute. %; \
  /echo %; \
  /echo -ah /listattrs %; \
  /echo %; \
  /echo /listattrs lists all the attributes - those set by /setattrs and %; \
  /echo those for specific channels - that you have defined. %; \
  /echo 

/def -i calvinhelp_rechelp = \
  /echo %; \
  /echo -ah /recchan <channel name> <lines> %; \
  /echo -ah /recchanwrap <width> <channel name> <lines> %; \
  /echo -ah /recdefault <lines> %; \
  /echo -ah /recwhisp <lines> %; \
  /echo -ah /recwhispname <person> <lines> %; \
  /echo %; \
  /echo All five macros let you recall specified output from the backlog. %; \
  /echo %; \
  /echo recchan recalls lines from a specified channel. %; \
  /echo %; \
  /echo recchanwrap does the same thing, wrapping to a specified width. %; \
  /echo %; \
  /echo recdefault recalls lines from your default channel. %; \
  /echo %; \
  /echo recwhisp recalls all whispered lines in the backlog range. %; \
  /echo %; \
  /echo recwhispname recalls whispered lines from a specific user. %; \
  /echo   It is case sensitive. %; \
  /echo %; \
  /echo

/def -i calvinhelp_chstats = \
  /echo %; \
  /echo -ah /setstat[nick][char][act] %; \
  /echo -ah /setstatdef %; \
  /echo -ah /setstatx %; \
  /echo %; \
  /echo All of these macros change the status line display. %; \
  /echo %; \
  /echo The macros of the form of the first line display your nick and %; \
  /echo default channel, default character and its default channel, and %; \
  /echo activity-tracked channels respectively if nick, char and act are %; \
  /echo present.  Note that this is order sensitive; /setstatnickact is %; \
  /echo valid but /setstatcharnick isn't.  Note that /setstatnickcharact %; \
  /echo sacrifices Tinyfugue's (Active: n) indicator. %; \
  /echo %; \
  /echo /setstatdef displays only your default channel.  Expected to be %; \
  /echo useful on pure-chat servers. %; \
  /echo %; \
  /echo /setstatx is designed for people who have the X Window System and %; \
  /echo are using the window captioning this script can provide.  It %; \
  /echo eliminates the world name and clock from the status bar in %; \
  /echo exchange for a very nicely spaced default channel, default %; \
  /echo character and activity tracking display. %; \
  /echo

/def -i calvinhelp_chrecon = \
  /echo %; \
  /echo -ah /recon <world> %; \
  /echo -ah /autocon <world> <nick> <channels> %; \
  /echo %; \
  /echo /recon reconnects to a Calvin chatserver, sets your nick to %; \
  /echo what it was when you were disconnected, and rejoins the %; \
  /echo channels you were on. %; \
  /echo %; \
  /echo /autocon connects to a Calvin chatserver, sets your nick to %; \
  /echo <nick>, and joins <channels> after logging in. %; \
  /echo

; ------------------------------- COMMAND ALIASES

; Characters

/def -i addchar = /calvinhelp_addchar %*
/def -i addnarr = /calvinhelp_addnarr %*
/def -i delchar = /calvinhelp_delchar %*
/def -i listchars = /calvinhelp_listchars %*

; Trustchars syntax

/def -i eagtrust = /calvinhelp_doeagtrust %*
/def -i noeagtrust = /calvinhelp_donoeagtrust %*
/def -i untrust = /calvinhelp_dountrust %*
/def -i nountrust = /calvinhelp_donountrust %*

/def -i eagtrustall = /calvinhelp_eagtrustall
/def -i eagtrustnone = /calvinhelp_eagtrustnone
/def -i untrustall = /calvinhelp_untrustall
/def -i untrustnone = /calvinhelp_untrustnone

/def -i showtrust = /calvinhelp_showalltrust

/def -i eagtrustdef = /echo % /eagtrustdef and /noeagtrustdef deprecated.
/def -i noeagtrustdef = /echo % /eagtrustdef and /noeagtrustdef deprecated.

; Attributes

/def -i chanattr = /calvinhelp_chanattr %*
/def -i nochanattr = /calvinhelp_nochanattr %*
/def -i setattrs = /calvinhelp_setattrs %*  
/def -i noattrs = /calvinhelp_noattrs %*
/def -i listattrs = /calvinhelp_listattrs %*
/def -i dosubs = /calvinhelp_dosubs

; Gags

/def -i addgag    = /calvinhelp_addgag %*
/def -i delgag    = /calvinhelp_delgag %*
/def -i gagmsg    = /calvinhelp_gagmsg %*
/def -i delgagmsg = /calvinhelp_delgagmsg %*
/def -i gagall    = /calvinhelp_gagall %*
/def -i delgagall = /calvinhelp_delgagall %*

; Recalls

/def -i recchan = /calvinhelp_recallchannel %*
/def -i recwhisp = /calvinhelp_recallwhisper %*
/def -i recwhispname = /calvinhelp_recallwhispername %*
/def -i recdef = /calvinhelp_recalldefault %*
/def -i recchanwrap = /calvinhelp_recchanwrap %*

; Status

/def -i setstatnickchar     = /calvinhelp_setstatnickchar %*
/def -i setstatchar         = /calvinhelp_setstatchar %*
/def -i setstatnick         = /calvinhelp_setstatnick %*
/def -i setstatact          = /calvinhelp_setstatact %*
/def -i setstatcharact      = /calvinhelp_setstatcharact %*
/def -i setstatnickact      = /calvinhelp_setstatnickact %*
/def -i setstatnickcharact  = /calvinhelp_setstatnickcharact %*
/def -i setstatdef          = /calvinhelp_setstatdef %*
/def -i setstatx            = /calvinhelp_setstatx %*

; Reconnect

/def -i recon = /calvinhelp_reconnect %*
/def -i autocon = /calvinhelp_autologin %*

; Help

/def -i chelp = /calvinhelp_calvinhelp %*
/def -i calvinhelp = /calvinhelp_calvinhelp %*
/def -i chchars = /calvinhelp_chchars %*
/def -i chattrs = /calvinhelp_chattrs %*
/def -i chstats = /calvinhelp_chstats %*
/def -i chrecon = /calvinhelp_chrecon %*
/def -i chrecs  = /calvinhelp_rechelp %*
/def -i chgag   = /calvinhelp_chgag   %*
/def -i chtrust = /calvinhelp_chtrust %*
/def -i rechelp = /calvinhelp_rechelp %*

/calvinhelp_dosubs
